@@ -54,6 +54,8 @@ from mw_python_sdk.utils import (
     parse_datetime,
     convert_to_dataset_file,
     generate_timestamped_string,
+    ProgressPercentage,
+    print_percent,
 )
 
 load_dotenv()  # take environment variables from local .env
@@ -166,27 +168,30 @@ def get_dataset(
         "x-kesci-token": token,
         "x-kesci-resource": dataset_id,
     }
-    params = {}
     if commit is not None:
-        params["Version"] = commit
-
-    response = requests.get(url, headers=headers, params=params, timeout=10)
+        url = f"{url}/versions/{commit}"
+    response = requests.get(url, headers=headers, timeout=10)
     if response.status_code == 200:
+        # 这里不确定Files和FilesStructure的关系
+        # 优先拿FilesStructure，并且拿Files的size补齐FilesStructure
+        # 以前的版本可能没有FilesStructure，所以这里需要兼容
         logger.debug("get datasets %s", response.json())
         files = convert_to_dataset_file(response.json().get("Files"))
         files_with_subpath = convert_to_dataset_file(
             response.json().get("FilesStructure")
         )
-        files_key_set = set([file.key for file in files_with_subpath])
+        files_key_set = dict({file.key: file for file in files_with_subpath})
         for file in files:
             if file.key not in files_key_set:
-                files_with_subpath.append(file)
+                files_key_set[file.key] = file
+            else:
+                files_key_set[file.key].size = file.size
         return Dataset(
             _id=response.json().get("_id"),
             title=response.json().get("Title"),
             short_description=response.json().get("ShortDescription"),
             folder_name=response.json().get("FolderName"),
-            files=files_with_subpath,
+            files=list(files_key_set.values()),
             commits=[
                 DatasetCommit(
                     commit.get("_id"),
@@ -291,6 +296,7 @@ def _get_update_token(token: Optional[str] = None) -> Optional[UploadInfo]:
             region=response.json().get("region"),
         )
     else:
+        print(response.text)
         response.raise_for_status()
         return None
 
@@ -337,10 +343,19 @@ def _upload_file(
             # https://docs.aws.amazon.com/AmazonS3/latest/userguide/qfacts.html
             multipart_config = TransferConfig(multipart_chunksize=5 * 1024 * 1024)
             s3.upload_file(
-                path_or_fileobj, bucket_name, object_key, Config=multipart_config
+                path_or_fileobj,
+                bucket_name,
+                object_key,
+                Config=multipart_config,
+                Callback=ProgressPercentage(path_or_fileobj),
             )
         else:
-            s3.put_object(Bucket=bucket_name, Key=object_key, Body=path_or_fileobj)
+            s3.put_object(
+                Bucket=bucket_name,
+                Key=object_key,
+                Body=path_or_fileobj,
+                Callback=ProgressPercentage(path_or_fileobj),
+            )
     except (IOError, s3.exceptions.S3UploadFailedError) as e:
         print(f"Error putting object '{object_key}' from bucket '{bucket_name}': {e}")
     return object_key
@@ -625,8 +640,11 @@ def _download_single_file(
                 f"File '{filename}' is being downloaded, please wait. If the downloading process is interrupted, please delete the lock file '{file_path}' manually."
             )
         with open(file_path, "wb") as file:
+            seen_so_far = 0
             for chunk in response.iter_content(chunk_size=chunk_size):
                 file.write(chunk)
+                seen_so_far += len(chunk)
+                print_percent(str(file_path), seen_so_far, total_size)
             file_path.rename(file_path.parent.joinpath(etag))
     slk_path.parent.mkdir(parents=True, exist_ok=True)
     slk_path.symlink_to(blob_path)
@@ -681,7 +699,10 @@ def download_dir(
             if sub_path is not None:
                 filepath_in_dataset = os.path.join(sub_path, filepath_in_dataset)
             # skip file if sub_dir is specified and filepath_in_dataset not in sub_dir
-            if sub_dir is not None and os.path.commonpath([sub_dir, filepath_in_dataset]) != sub_dir:
+            if (
+                sub_dir is not None
+                and os.path.commonpath([sub_dir, filepath_in_dataset]) != sub_dir
+            ):
                 continue
             key = path_to_key[filepath_in_dataset]
             mw_cached_root_dir = os.environ.get("MW_CACHED_ROOT_DIR", "/var/mw")
@@ -757,6 +778,40 @@ def download_dir(
         return Path(local_dir)
 
 
+def _get_download_url(
+    dataset_id: str,
+    filename: str,
+    commit: str,
+    token: str,
+):
+    url = f"{HEYWHALE_SITE}/api/datasets/{dataset_id}/downloadUrl"
+    headers = {
+        "x-kesci-token": token,
+    }
+    params = {"Version": commit}
+    response = requests.get(url, headers=headers, params=params, timeout=10)
+    download_url: str = ""
+    if response.status_code == 200:
+        files = response.json().get("files")
+        logger.debug("get dataset files through api: %s", files)
+        for file_json in files:
+            sub_path = (
+                file_json.get("SubPath") if file_json.get("SubPath") is not None else ""
+            )
+            if (
+                os.path.join(sub_path, file_json.get("Name")) + file_json.get("Ext")
+                == filename
+            ):
+                download_url = file_json.get("Url")
+                break
+        else:
+            raise ValueError(f"File '{filename}' not found in dataset '{dataset_id}'.")
+    else:
+        print(response.text)
+        response.raise_for_status()
+    return download_url
+
+
 def download_file(
     dataset_id: str,
     filename: str,
@@ -793,33 +848,11 @@ def download_file(
 
     bucket_name = upload_info.bucket
 
-    url = f"{HEYWHALE_SITE}/api/datasets/{dataset_id}/downloadUrl"
-    headers = {
-        "x-kesci-token": token,
-    }
     if commit is None:
         commit = dataset_detail.commits[-1].get_id()
-    params = {"Version": commit}
-    response = requests.get(url, headers=headers, params=params, timeout=10)
-    download_url: str = ""
-    if response.status_code == 200:
-        files = response.json().get("files")
-        logger.debug("get dataset files through api: %s", files)
-        for file_json in files:
-            sub_path = (
-                file_json.get("SubPath") if file_json.get("SubPath") is not None else ""
-            )
-            if (
-                os.path.join(sub_path, file_json.get("Name")) + file_json.get("Ext")
-                == filename
-            ):
-                download_url = file_json.get("Url")
-                break
-        else:
-            raise ValueError(f"File '{filename}' not found in dataset '{dataset_id}'.")
-    else:
-        print(response.text)
-        response.raise_for_status()
+
+    download_url: str = _get_download_url(dataset_id, filename, commit, token)
+
     key = path_to_key[filename]
     mw_cached_root_dir = os.environ.get("MW_CACHED_ROOT_DIR", "/var/mw")
     sock_path = os.path.join(mw_cached_root_dir, "cached.sock")
@@ -1036,6 +1069,7 @@ def _create_dataset(
             updated_at=parse_datetime(document.get("UpdateDate")),
         )
     else:
+        print(response.text)
         response.raise_for_status()
         return None
 
